package BlankOnDev::Version;
use strict;
use warnings FATAL => 'all';

# Version :
our $VERSION = '0.1004';;
our $appVer = '0.1';
our $SubVer = '04';

# boidev.pl version :
our $boidev_ver = '0.1001';

# bzr2git.pl version :
our $bzr2git_ver = '0.1000';
1;
#__END__
#